
from . import scores


def configure(config, bp, scorer_map):

    @bp.route("/", methods=["GET"])
    def index():
        return "Hi!  I'm ORES.  :D"

    bp = scores.configure(config, bp, scorer_map)

    return bp



from flask.ext.jsonpify import jsonify


def error(status, code, message):
    return jsonify({'error': {'code': code, 'message': message}}), status

def not_implemented(message=None):
    return error(501, 'not implemented',
                 message or "Route not implemented yet.")

def bad_request(message):
    return error(400, 'bad request', message)

def forbidden(message=None):
    return error(403, 'forbidden',
                 message or "This request requires authentication.")

def not_found(message=None):
    return error(404, 'not found',
                 message or "Nothing found at this location.")




class ParamError(Exception): pass


def read_param(request, param, default=None, type=str):
    try:
        value = request.args.get(param, request.form.get(param, default))
        return type(value.strip())
    except (ValueError, TypeError) as e:
        raise ParamError("Could not interpret {0}. {1}"
                         .format(param, str(e)))


def read_bar_split_param(request, param, default=None, type=str):
    values = read_param(request, param, default=default)
    if values is None:
        return []
    return [type(value) for value in values.split("|")]



from revscoring.features import (diff, page, parent_revision,
                                 previous_user_revision, revision, user)
from revscoring.features.modifiers import log

from . import generic

damaging = [
    log(diff.added_symbolic_chars_ratio + 1),
    log(diff.chars_added + 1),
    log(diff.chars_removed + 1),
    diff.longest_repeated_char_added,
    diff.longest_token_added,
    log(diff.markup_chars_added + 1),
    log(diff.markup_chars_removed + 1),
    log(diff.numeric_chars_added + 1),
    log(diff.numeric_chars_removed + 1),
    diff.proportion_of_chars_added,
    diff.proportion_of_chars_removed,
    diff.proportion_of_markup_chars_added,
    diff.proportion_of_numeric_chars_added,
    diff.proportion_of_symbolic_chars_added,
    diff.proportion_of_uppercase_chars_added,
    log(diff.segments_added + 1),
    log(diff.segments_removed + 1),
    log(diff.symbolic_chars_added + 1),
    log(diff.symbolic_chars_removed + 1),
    log(diff.uppercase_chars_added + 1),
    log(diff.uppercase_chars_removed + 1),
    log(diff.words_added + 1),
    log(diff.words_removed + 1),
    diff.bytes_changed + 1,
    diff.bytes_changed_ratio,
    page.is_content_namespace,
    parent_revision.was_same_user,
    log(parent_revision.words + 1),
    log(user.age + 1),
    user.is_anon,
    user.is_bot,
    log(diff.added_badwords_ratio + 1),
    log(diff.badwords_added + 1),
    log(diff.badwords_removed + 1),
    diff.proportion_of_badwords_added,
    diff.proportion_of_badwords_removed
]

good_faith = generic.good_faith + [
    log(diff.added_badwords_ratio + 1),
    log(diff.badwords_added + 1),
    log(diff.badwords_removed + 1),
    log(diff.proportion_of_badwords_added + 1),
    log(diff.proportion_of_badwords_removed + 1),
    log(diff.removed_badwords_ratio + 1),
    log(parent_revision.badwords + 1),
    log(parent_revision.proportion_of_badwords + 1),
    log(revision.badwords + 1),
    log(revision.proportion_of_badwords + 1)
]



import json
import os

from flask import Blueprint, Flask, request
from revscoring.scorers import Scorer

from . import routes


def configure(config):
    app = Flask("ores")
    app.config["APPLICATION_ROOT"] = config['application_root']

    bp = Blueprint('ores', __name__)

    scorer_map = {wiki:Scorer.from_config(config, wiki)
                  for wiki in config['scorers']}

    bp = routes.configure(config, bp, scorer_map)

    app.register_blueprint(bp, url_prefix=config['url_prefix'])

    return app



import traceback
from collections import defaultdict

from flask import request
from flask.ext.jsonpify import jsonify

from .. import responses
from ..util import ParamError, read_bar_split_param


def configure(config, bp, scorer_map, cache):

    # /scores/
    @bp.route("/scores/", methods=["GET"])
    def scores():
        scorers = [wiki for wiki in scorer_map]

        scorers.sort()

        return jsonify({'scorers': scorers})

    # /scores/enwiki/?models=reverted&revids=456789|4567890
    @bp.route("/scores/<wiki>/", methods=["GET"])
    def scores_revisions(wiki):

        # Check to see if we have the wiki
        if wiki in scorer_map:
            scorer = scorer_map[wiki]
        else:
            return responses.not_found("No scorers available for {0}" \
                                       .format(wiki))

        # If no model is specified, return information on available models
        if "models" not in request.args:
            # Return the models that we have
            return jsonify({"models": list(scorer_map[wiki].model_map.keys())})

        # Read the params
        try:
            model_names = set(read_bar_split_param(request, "models", type=str))
            rev_ids = set(read_bar_split_param(request, "revids", type=int))
        except ParamError as e:
            return responses.bad_request(str(e))

        # Check that all requested models are available
        missing_models = model_names - scorer.models.keys()
        if len(missing_models) > 0:
            message = ("Models {0} is not available for {1}.  " +
                       "Available models: {2}") \
                      .format(list(missing_models), wiki,
                              list(scorer.models.keys())
            return responses.bad_request(message)

        # Lookup cached scores
        scores = cache.lookup_many(rev_ids, wiki, model_names)
        missing_rev_ids = rev_ids - scores.keys()

        # TODO: Pre-cache datasources for missing_rev_ids and distribute
        # processing to celery
        # For now, just process missing rev_ids in batch
        scores = {rev_id:score for rev_id, score in
                  zip(rev_ids, scorer.score_many(missing_rev_ids,
                                                 models=model_names))}

        return jsonify(scores)

    return bp



from revscoring.languages import portuguese
from revscoring.scorers import LinearSVCModel

from ..feature_lists.ptwiki import damaging

damaging_linear_svc = LinearSVCModel(damaging, language=portuguese)




class Cache:

    def lookup(wiki, model, rev_id):
        """
        Returns a pre-cached score
        """
        raise NotImplementedError()

    def store(wiki, model, rev_id, score):
        """
        Caches a new score
        """
        raise NotImplementedError()



r"""
Gathers the reverted status for a set of revisions and
prints a TSV to stdout of the format:

<rev_id>\t<reverted>

Usage:
    label_reverted -h | --help
    label_reverted --api=<url> [--revert-radius=<revs>]
                               [--revert-window=<secs>]
                               [--rev-pages=<path>]
                               [--rev-reverteds=<path>]

Options:
    -h --help               Prints out this documentation
    <features>              Classpath to a list of features to extract
    --api=<url>             The url of the API to use to extract features
    --revert-radius=<revs>  The maximum amount of revisions that a reverting
                            edit can revert [default: 15]
    --revert-window=<secs>  The maximum amount of time to wait for a revision to
                            be reverted [default: 172800]
    --rev-pages=<path>      The location of a file containing rev_ids and
                            page_ids to extract. [default: <stdin>]
    --rev-reverteds=<path>  The location to write output to. [default: <stdout>]
"""
import sys
import traceback

import docopt
from mw import api
from mw.lib import reverts
from revscoring.extractors import APIExtractor

from .util import import_from_path


def main(argv=None):
    args = docopt.docopt(__doc__, argv=argv)

    revert_radius = int(args['--revert-radius'])
    revert_window = int(args['--revert-window'])

    if args['--rev-pages'] == "<stdin>":
        rev_pages = read_rev_pages(sys.stdin)
    else:
        rev_pages = read_rev_pages(open(args['--rev-pages']))

    if args['--rev-reverteds'] == "<stdout>":
        rev_reverteds = sys.stdout
    else:
        rev_reverteds = open(args['--rev-reverteds'], "w")

    api_url = args['--api']
    session = api.Session(api_url, user_agent="ORES revert labeling utility")

    run(rev_pages, rev_reverteds, session, revert_radius, revert_window)

def read_rev_pages(f):
    first_line_parts = f.readline().split("\t")
    if first_line_parts[0] != "rev_id":
        if len(first_line_parts) == 1:
            rev_id = first_line_parts[0]
            yield int(rev_id), None
        elif len(first_line_parts) == 2:
            rev_id, page_id = first_line_parts
            yield int(rev_id), int(page_id)

    for line in f:
        parts = line.strip().split('\t')

        if len(parts) == 1:
            rev_id = parts[0]
            yield int(rev_id), None
        elif len(parts) == 2:
            rev_id, page_id = parts
            yield int(rev_id), int(page_id)

def run(rev_pages, rev_reverteds, session, revert_radius, revert_window):


    for rev_id, page_id in rev_pages:
        try:
            # Detect reverted status
            revert = reverts.api.check(session, rev_id,
                                       page_id=page_id, # Could be None
                                       radius=revert_radius,
                                       window=revert_window)

            reverted = revert is not None

            if reverted:
                sys.stderr.write("r");sys.stderr.flush()
            else:
                sys.stderr.write(".");sys.stderr.flush()

            # Print out row
            rev_reverteds.write('\t'.join(str(v) for v in [rev_id, reverted]))
            rev_reverteds.write("\n")

        except KeyboardInterrupt:
            sys.stderr.write("\n^C Caught.  Exiting...")
            break

        except:
            sys.stderr.write(traceback.format_exc())
            sys.stderr.write("\n")

    sys.stderr.write("\n")


if __name__ == "__main__":
    main()



from revscoring.features import (
    diff, page, parent_revision, previous_user_revision, revision, user
)
from revscoring.features.modifiers import log
from . import generic

damaging = generic.damaging
good_faith = generic.good_faith




"""
This script provides access to a set of utilities for ORES

* label_reverted -- Labels reverted revisions
* dev_server -- Starts a development WSGI server

Usage:
    ores (-h | --help)
    ores <utility> [-h | --help]

Options:
    -h | --help  Shows this documentation
    <utility>    The name of the utility to run
"""
import sys
import traceback
from importlib import import_module

import docopt


USAGE = """Usage:
    ores (-h | --help)
    ores <utility> [-h | --help]\n"""


def main():

    if len(sys.argv) < 2:
        sys.stderr.write(USAGE)
        sys.exit(1)
    elif sys.argv[1] in ("-h", "--help"):
        sys.stderr.write(__doc__ + "\n")
        sys.exit(1)
    elif sys.argv[1][:1] == "-":
        sys.stderr.write(USAGE)
        sys.exit(1)

    module_name = sys.argv[1]
    try:
        sys.path.insert(0, ".")
        module = import_module(".utilities." + module_name, package="ores")
    except ImportError:
        sys.stderr.write(traceback.format_exc())
        sys.stderr.write("Could not find utility {0}.\n".format(module_name))
        sys.exit(1)

    module.main(sys.argv[2:])



import json

import redis

from .cache import Cache


class Redis(Cache):

    def __init__(self, redis, prefix=""):
        self.redis = redis
        self.prefix = str(prefix)

    def lookup(self, wiki, model, rev_id):
        wiki = str(wiki)
        model = str(model)
        rev_id = str(rev_id)
        try:
            value = self.redis.get(":".join([self.prefix, wiki, model, rev_id]))
            return json.loads(value)
        except KeyError:
            return None

    def store(self, wiki, model, rev_id, score):
        wiki = str(wiki)
        model = str(model)
        rev_id = str(rev_id)
        score = json.dumps(score)

        self.redis.put(":".join([self.prefix, wiki, model, rev_id],
                       score)

    @classmethod
    def from_parameters(cls, host, port, prefix=""):
        return cls(redis.connect(host=host, port=port), prefix=prefix)



import os

from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def requirements(fname):
    return [line.strip()
            for line in open(os.path.join(os.path.dirname(__file__), fname))]

setup(
    name="ores",
    version="0.1.0", # Update in ores/__init__.py too.
    author="Aaron Halfaker",
    author_email="ahalfaker@wikimedia.org",
    description=("A webserver for hosting scoring services."),
    license="MIT",
    entry_points = {
        'console_scripts': [
            'ores = ores.ores:main',
        ],
    },
    url="https://github.com/halfak/Objective-Revision-Evaluation-Service",
    packages=find_packages(),
    long_description=read('README.md'),
    install_requires=requirements("requirements.txt"),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)



from revscoring.features import (diff, page, parent_revision,
                                 previous_user_revision, revision, user)
from revscoring.features.modifiers import log

generic_ratio = [
    diff.added_markup_chars_ratio,
    diff.added_number_chars_ratio,
    diff.added_symbolic_chars_ratio,
    diff.added_uppercase_chars_ratio,
    diff.bytes_changed_ratio,
    log(diff.proportion_of_chars_added + 1),
    log(diff.proportion_of_chars_removed + 1),
    log(diff.proportion_of_markup_chars_added + 1),
    log(diff.proportion_of_numeric_chars_added + 1),
    log(diff.proportion_of_symbolic_chars_added + 1),
    log(diff.proportion_of_uppercase_chars_added + 1),
    parent_revision.proportion_of_markup_chars,
    parent_revision.proportion_of_numeric_chars,
    parent_revision.proportion_of_symbolic_chars,
    parent_revision.proportion_of_uppercase_chars,
    revision.proportion_of_markup_chars,
    revision.proportion_of_numeric_chars,
    revision.proportion_of_symbolic_chars,
    revision.proportion_of_templated_references,
    revision.proportion_of_uppercase_chars
]

generic_added = [
    log(diff.chars_added + 1),
    log(diff.longest_repeated_char_added + 1),
    log(diff.longest_token_added + 1),
    log(diff.markup_chars_added + 1),
    log(diff.numeric_chars_added + 1),
    log(diff.segments_added + 1),
    log(diff.symbolic_chars_added + 1),
    log(diff.uppercase_chars_added + 1),
    log(diff.words_added + 1)
]

generic_removed = [
    log(diff.chars_removed + 1),
    log(diff.markup_chars_removed + 1),
    log(diff.numeric_chars_removed + 1),
    log(diff.segments_removed + 1),
    log(diff.symbolic_chars_removed + 1),
    log(diff.uppercase_chars_removed + 1),
    log(diff.words_removed + 1)
]

generic_count = [
    parent_revision.bytes,
    log(parent_revision.chars + 1),
    log(parent_revision.markup_chars + 1),
    log(parent_revision.numeric_chars + 1),
    log(parent_revision.revision_bytes + 1),
    log(parent_revision.symbolic_chars + 1),
    log(parent_revision.uppercase_chars + 1),
    log(parent_revision.words + 1),
    log(revision.chars + 1),
    log(revision.markup_chars + 1),
    log(revision.numeric_chars + 1),
    log(revision.words + 1),
    log(revision.symbolic_chars + 1),
    log(revision.uppercase_chars + 1)
]

generic_wikimarkup = [
    page.is_mainspace,
    page.is_content_namespace,
    log(revision.cite_templates + 1),
    log(revision.has_custom_comment + 1),
    log(revision.has_section_comment + 1),
    log(revision.image_links + 1),
    log(revision.infobox_templates + 1),
    log(revision.internal_links + 1),
    log(revision.level_1_headings + 1),
    log(revision.level_2_headings + 1),
    log(revision.level_3_headings + 1),
    log(revision.level_4_headings + 1),
    log(revision.level_5_headings + 1),
    log(revision.level_6_headings + 1),
    log(revision.ref_tags + 1),
    log(revision.templates + 1)
]

generic_other = [
    diff.bytes_changed,
    log(page.age + 1),
    log(parent_revision.seconds_since + 1),
    parent_revision.was_same_user,
    log(parent_revision.words + 1),
    log(previous_user_revision.seconds_since + 1),
    log(revision.day_of_week + 1),
    log(user.age + 1),
    log(previous_user_revision.seconds_since + 1),
    user.is_anon,
    user.is_bot
]

generic_unused = [
    revision.bytes,
    log(revision.category_links + 1),
    log(revision.hour_of_day + 1),
]

generic_minimalist = [
    log(diff.added_markup_chars_ratio + 1),
    log(diff.added_number_chars_ratio + 1),
    log(diff.added_symbolic_chars_ratio + 1),
    log(diff.added_uppercase_chars_ratio + 1),
    log(diff.chars_added + 1),
    log(diff.chars_removed + 1),
    log(diff.longest_repeated_char_added + 1),
    log(diff.longest_token_added + 1),
    log(diff.markup_chars_added + 1),
    log(diff.markup_chars_removed + 1),
    log(diff.numeric_chars_added + 1),
    log(diff.numeric_chars_removed + 1),
    diff.proportion_of_chars_added,
    diff.proportion_of_chars_removed,
    diff.proportion_of_markup_chars_added,
    diff.proportion_of_numeric_chars_added,
    diff.proportion_of_symbolic_chars_added,
    diff.proportion_of_uppercase_chars_added,
    log(diff.segments_added + 1),
    log(diff.segments_removed + 1),
    log(diff.symbolic_chars_added + 1),
    log(diff.symbolic_chars_removed + 1),
    log(diff.uppercase_chars_added + 1),
    log(diff.uppercase_chars_removed + 1),
    log(diff.words_added + 1),
    log(diff.words_removed + 1),
    diff.bytes_changed,
    diff.bytes_changed_ratio,
    page.is_content_namespace,
    parent_revision.was_same_user,
    log(user.age + 1),
    user.is_anon,
    user.is_bot
]

#generic = generic_ratio + generic_added + generic_removed + generic_count + generic_wikimarkup + generic_other
generic = generic_minimalist

damaging = generic

good_faith = generic



"""
Starts a development web server.

Usage:
    dev_server (-h | --help)
    dev_server [--port=<num>] [--verbose] [--config=<path>]

Options:
    -h --help        Print this documentation
    --port=<num>     The port number to start the server on [default: 8080]
    --config=<path>  The path to a yaml config file
                     [default: config/ores-localdev.yaml]
    --verbose        Print logging information
"""
import docopt

import yamlconf

from ..wsgi import application


def main(argv=None):
    args = docopt.docopt(__doc__, argv=argv)

    config = yamlconf.load(open(args['--config']))

    app = application.configure(config)
    app.run(host="0.0.0.0",
            port=int(args['--port']),
            debug=True,
            ssl_context="adhoc")
